@@ -67,7 +67,7 @@ void TestDragAndDrop::init()
     using namespace KWayland::Client;
     delete m_display;
     m_display = new Display(this);
-    m_display->setSocketName(s_socketName);
+    m_display->addSocketName(s_socketName);
     m_display->start();
     QVERIFY(m_display->isRunning());
 
@@ -77,15 +77,13 @@ void TestDragAndDrop::init()
     QVERIFY(connectedSpy.isValid());
     m_connection->setSocketName(s_socketName);
 
-    m_compositorInterface = m_display->createCompositor(m_display);
-    m_seatInterface = m_display->createSeat(m_display);
+    m_compositorInterface = new CompositorInterface(m_display, m_display);
+    m_seatInterface = new SeatInterface(m_display, m_display);
     m_seatInterface->setHasPointer(true);
     m_seatInterface->setHasTouch(true);
     m_seatInterface->create();
     QVERIFY(m_seatInterface->isValid());
-    m_dataDeviceManagerInterface = m_display->createDataDeviceManager(m_display);
-    m_dataDeviceManagerInterface->create();
-    QVERIFY(m_dataDeviceManagerInterface->isValid());
+    m_dataDeviceManagerInterface = new DataDeviceManagerInterface(m_display, m_display);
     m_display->createShm();
 
     m_thread = new QThread(this);
@@ -247,6 +245,7 @@ void TestDragAndDrop::testPointerDragAndDrop()
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().count(), 1);
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().first().name(), QStringLiteral("text/plain"));
     QTRY_COMPARE(offer->sourceDragAndDropActions(), DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move);
+    offer->accept(QStringLiteral("text/plain"), dragEnteredSpy.last().at(0).toUInt());
     offer->setDragAndDropActions(DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move, DataDeviceManager::DnDAction::Move);
     QVERIFY(offerActionChangedSpy.wait());
     QCOMPARE(offerActionChangedSpy.count(), 1);
@@ -346,6 +345,7 @@ void TestDragAndDrop::testTouchDragAndDrop()
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().count(), 1);
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().first().name(), QStringLiteral("text/plain"));
     QTRY_COMPARE(offer->sourceDragAndDropActions(), DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move);
+    offer->accept(QStringLiteral("text/plain"), dragEnteredSpy.last().at(0).toUInt());
     offer->setDragAndDropActions(DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move, DataDeviceManager::DnDAction::Move);
     QVERIFY(offerActionChangedSpy.wait());
     QCOMPARE(offerActionChangedSpy.count(), 1);
@@ -440,6 +440,7 @@ void TestDragAndDrop::testDragAndDropWithCancelByDestroyDataSource()
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().count(), 1);
     QCOMPARE(m_dataDevice->dragOffer()->offeredMimeTypes().first().name(), QStringLiteral("text/plain"));
     QTRY_COMPARE(offer->sourceDragAndDropActions(), DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move);
+    offer->accept(QStringLiteral("text/plain"), dragEnteredSpy.last().at(0).toUInt());
     offer->setDragAndDropActions(DataDeviceManager::DnDAction::Copy | DataDeviceManager::DnDAction::Move, DataDeviceManager::DnDAction::Move);
     QVERIFY(offerActionChangedSpy.wait());
     QCOMPARE(offerActionChangedSpy.count(), 1);
@@ -543,11 +544,11 @@ void TestDragAndDrop::testPointerEventsIgnored()
     m_seatInterface->setPointerPos(QPointF(50, 50));
 
     // last but not least, simulate the drop
-    QSignalSpy droppedSpy(m_dataDevice, &DataDevice::dropped);
-    QVERIFY(droppedSpy.isValid());
+    QSignalSpy cancelledSpy(m_dataSource, &DataSource::cancelled);
+    QVERIFY(cancelledSpy.isValid());
     m_seatInterface->setTimestamp(timestamp++);
     m_seatInterface->pointerButtonReleased(1);
-    QVERIFY(droppedSpy.wait());
+    QVERIFY(cancelledSpy.wait());
 
     // all the changes should have been ignored
     QCOMPARE(axisSpy.count(), 1);
