@@ -1,5 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2020 David Edmundson <davidedmundson@kde.org>
+    SPDX-FileCopyrightText: 2020 Vlad Zahorodnii <vlad.zahorodnii@kde.org>
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
@@ -21,6 +23,37 @@ class DataSourceInterface;
 class AbstractDataSource;
 class SeatInterface;
 class SurfaceInterface;
+class DataDeviceInterfacePrivate;
+class DragAndDropIconPrivate;
+
+/**
+ * The DragAndDropIcon class represents a drag-and-drop icon.
+ *
+ * Note that the lifetime of the drag-and-drop icon is bound to the lifetime of the underlying
+ * icon surface.
+ */
+class KWAYLANDSERVER_EXPORT DragAndDropIcon : public QObject
+{
+    Q_OBJECT
+
+public:
+    ~DragAndDropIcon() override;
+
+    /**
+     * Returns the position of the icon relative to the cursor's hotspot.
+     */
+    QPoint position() const;
+
+    /**
+     * Returns the underlying icon surface. This function always returns a valid surface.
+     */
+    SurfaceInterface *surface() const;
+
+private:
+    explicit DragAndDropIcon(SurfaceInterface *surface, QObject *parent = nullptr);
+    friend class DataDeviceInterfacePrivate;
+    QScopedPointer<DragAndDropIconPrivate> d;
+};
 
 /**
  * @brief DataDeviceInterface allows clients to share data by copy-and-paste and drag-and-drop.
@@ -33,7 +66,7 @@ class SurfaceInterface;
  * @see SeatInterface
  * @see DataSourceInterface
  **/
-class KWAYLANDSERVER_EXPORT DataDeviceInterface : public Resource
+class KWAYLANDSERVER_EXPORT DataDeviceInterface : public QObject
 {
     Q_OBJECT
 public:
@@ -42,7 +75,11 @@ public:
     SeatInterface *seat() const;
     DataSourceInterface *dragSource() const;
     SurfaceInterface *origin() const;
-    SurfaceInterface *icon() const;
+    /**
+     * Returns the additional icon attached to the cursor during a drag-and-drop operation.
+     * This function returns @c null if no drag-and-drop icon has been attached.
+     */
+    DragAndDropIcon *icon() const;
 
     /**
      * @returns the serial of the implicit grab which started the drag
@@ -78,17 +115,19 @@ public:
      **/
     void updateProxy(SurfaceInterface *remote);
 
+    wl_client *client();
+
 Q_SIGNALS:
+    void aboutToBeDestroyed();
     void dragStarted();
     void selectionChanged(KWaylandServer::DataSourceInterface*);
     void selectionCleared();
 
 private:
-    friend class DataDeviceManagerInterface;
-    explicit DataDeviceInterface(SeatInterface *seat, DataDeviceManagerInterface *parent, wl_resource *parentResource);
-
-    class Private;
-    Private *d_func() const;
+    friend class DataDeviceManagerInterfacePrivate;
+    explicit DataDeviceInterface(SeatInterface *seat, wl_resource *resource);
+    QScopedPointer<DataDeviceInterfacePrivate> d;
+    friend class DataDeviceInterfacePrivate;
 };
 
 }
