@@ -5,21 +5,25 @@
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 #include "abstract_data_source.h"
+#include "display_p.h"
 #include "seat_interface.h"
 #include "seat_interface_p.h"
 #include "display.h"
 #include "datadevice_interface.h"
+#include "datadevice_interface_p.h"
 #include "datasource_interface.h"
 #include "datacontroldevice_v1_interface.h"
 #include "datacontrolsource_v1_interface.h"
 #include "keyboard_interface.h"
 #include "keyboard_interface_p.h"
+#include "logging.h"
 #include "pointer_interface.h"
 #include "pointer_interface_p.h"
 #include "primaryselectiondevice_v1_interface.h"
 #include "primaryselectionsource_v1_interface.h"
 #include "surface_interface.h"
-#include "textinput_interface_p.h"
+#include "textinput_v2_interface_p.h"
+#include "textinput_v3_interface_p.h"
 // Qt
 #include <QFile>
 // Wayland
@@ -46,6 +50,8 @@ SeatInterface::Private::Private(SeatInterface *q, Display *display)
     : Global::Private(display, &wl_seat_interface, s_version)
     , q(q)
 {
+    textInputV2 = new TextInputV2Interface(q);
+    textInputV3 = new TextInputV3Interface(q);
 }
 
 #ifndef K_DOXYGEN
@@ -76,11 +82,20 @@ SeatInterface::SeatInterface(Display *display, QObject *parent)
     connect(this, &SeatInterface::hasPointerChanged,  this, sendCapabilitiesAll);
     connect(this, &SeatInterface::hasKeyboardChanged, this, sendCapabilitiesAll);
     connect(this, &SeatInterface::hasTouchChanged,    this, sendCapabilitiesAll);
+
+    DisplayPrivate *displayPrivate = DisplayPrivate::get(d->display);
+    displayPrivate->seats.append(this);
 }
 
 SeatInterface::~SeatInterface()
 {
     Q_D();
+
+    if (d->display) {
+        DisplayPrivate *displayPrivate = DisplayPrivate::get(d->display);
+        displayPrivate->seats.removeOne(this);
+    }
+
     while (!d->resources.isEmpty()) {
         wl_resource_destroy(d->resources.takeLast());
     }
@@ -132,20 +147,6 @@ void SeatInterface::Private::updatePointerButtonState(quint32 button, Pointer::S
     it.value() = state;
 }
 
-bool SeatInterface::Private::updateKey(quint32 key, Keyboard::State state)
-{
-    auto it = keys.states.find(key);
-    if (it == keys.states.end()) {
-        keys.states.insert(key, state);
-        return true;
-    }
-    if (it.value() == state) {
-        return false;
-    }
-    it.value() = state;
-    return true;
-}
-
 void SeatInterface::Private::sendName(wl_resource *r)
 {
     if (wl_resource_get_version(r) < WL_SEAT_NAME_SINCE_VERSION) {
@@ -169,6 +170,11 @@ void SeatInterface::Private::sendCapabilities(wl_resource *r)
     wl_seat_send_capabilities(r, capabilities);
 }
 
+quint32 SeatInterface::Private::nextSerial() const
+{
+    return display->nextSerial();
+}
+
 SeatInterface::Private *SeatInterface::Private::cast(wl_resource *r)
 {
     return r ? reinterpret_cast<SeatInterface::Private*>(wl_resource_get_user_data(r)) : nullptr;
@@ -232,11 +238,6 @@ QVector<PointerInterface *> SeatInterface::Private::pointersForSurface(SurfaceIn
     return interfacesForSurface(surface, pointers);
 }
 
-QVector<KeyboardInterface *> SeatInterface::Private::keyboardsForSurface(SurfaceInterface *surface) const
-{
-    return interfacesForSurface(surface, keyboards);
-}
-
 QVector<TouchInterface *> SeatInterface::Private::touchsForSurface(SurfaceInterface *surface) const
 {
     return interfacesForSurface(surface, touchs);
@@ -244,12 +245,16 @@ QVector<TouchInterface *> SeatInterface::Private::touchsForSurface(SurfaceInterf
 
 QVector<DataDeviceInterface *> SeatInterface::Private::dataDevicesForSurface(SurfaceInterface *surface) const
 {
-    return interfacesForSurface(surface, dataDevices);
-}
-
-TextInputInterface *SeatInterface::Private::textInputForSurface(SurfaceInterface *surface) const
-{
-    return interfaceForSurface(surface, textInputs);
+    if (!surface) {
+        return {};
+    }
+    QVector<DataDeviceInterface *> primarySelectionDevices;
+    for (auto it = dataDevices.constBegin(); it != dataDevices.constEnd(); ++it) {
+        if ((*it)->client() == *surface->client()) {
+            primarySelectionDevices << *it;
+        }
+    }
+    return primarySelectionDevices;
 }
 
 void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
@@ -258,10 +263,9 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
     dataDevices << dataDevice;
     auto dataDeviceCleanup = [this, dataDevice] {
         dataDevices.removeOne(dataDevice);
-        keys.focus.selections.removeOne(dataDevice);
+        globalKeyboard.focus.selections.removeOne(dataDevice);
     };
     QObject::connect(dataDevice, &QObject::destroyed, q, dataDeviceCleanup);
-    QObject::connect(dataDevice, &Resource::unbound, q, dataDeviceCleanup);
     QObject::connect(dataDevice, &DataDeviceInterface::selectionChanged, q,
         [this, dataDevice] {
             updateSelection(dataDevice);
@@ -299,20 +303,15 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
             }
             drag.source = dataDevice;
             drag.sourcePointer = interfaceForSurface(originSurface, pointers);
-            drag.destroyConnection = QObject::connect(dataDevice, &QObject::destroyed, q,
+            drag.destroyConnection = QObject::connect(dataDevice, &DataDeviceInterface::aboutToBeDestroyed, q,
                 [this] {
-                    endDrag(display->nextSerial());
+                    cancelDrag(display->nextSerial());
                 }
             );
             if (dataDevice->dragSource()) {
-                drag.dragSourceDestroyConnection = QObject::connect(dataDevice->dragSource(), &Resource::aboutToBeUnbound, q,
+                drag.dragSourceDestroyConnection = QObject::connect(dataDevice->dragSource(), &AbstractDataSource::aboutToBeDestroyed, q,
                     [this] {
-                        const auto serial = display->nextSerial();
-                        if (drag.target) {
-                            drag.target->updateDragTarget(nullptr, serial);
-                            drag.target = nullptr;
-                        }
-                        endDrag(serial);
+                        cancelDrag(display->nextSerial());
                     }
                 );
             } else {
@@ -324,10 +323,10 @@ void SeatInterface::Private::registerDataDevice(DataDeviceInterface *dataDevice)
         }
     );
     // is the new DataDevice for the current keyoard focus?
-    if (keys.focus.surface) {
+    if (globalKeyboard.focus.surface) {
         // same client?
-        if (keys.focus.surface->client() == dataDevice->client()) {
-            keys.focus.selections.append(dataDevice);
+        if (*globalKeyboard.focus.surface->client() == dataDevice->client()) {
+            globalKeyboard.focus.selections.append(dataDevice);
             if (currentSelection) {
                 dataDevice->sendSelection(currentSelection);
             }
@@ -376,7 +375,7 @@ void SeatInterface::Private::registerPrimarySelectionDevice(PrimarySelectionDevi
     primarySelectionDevices << primarySelectionDevice;
     auto dataDeviceCleanup = [this, primarySelectionDevice] {
         primarySelectionDevices.removeOne(primarySelectionDevice);
-        keys.focus.primarySelections.removeOne(primarySelectionDevice);
+        globalKeyboard.focus.primarySelections.removeOne(primarySelectionDevice);
     };
     QObject::connect(primarySelectionDevice, &QObject::destroyed, q, dataDeviceCleanup);
     QObject::connect(primarySelectionDevice, &PrimarySelectionDeviceV1Interface::selectionChanged, q,
@@ -390,10 +389,10 @@ void SeatInterface::Private::registerPrimarySelectionDevice(PrimarySelectionDevi
         }
     );
     // is the new DataDevice for the current keyoard focus?
-    if (keys.focus.surface) {
+    if (globalKeyboard.focus.surface) {
         // same client?
-        if (*keys.focus.surface->client() == primarySelectionDevice->client()) {
-            keys.focus.primarySelections.append(primarySelectionDevice);
+        if (*globalKeyboard.focus.surface->client() == primarySelectionDevice->client()) {
+            globalKeyboard.focus.primarySelections.append(primarySelectionDevice);
             if (currentPrimarySelection) {
                 primarySelectionDevice->sendSelection(currentPrimarySelection);
             }
@@ -401,44 +400,44 @@ void SeatInterface::Private::registerPrimarySelectionDevice(PrimarySelectionDevi
     }
 }
 
-void SeatInterface::Private::registerTextInput(TextInputInterface *ti)
+void SeatInterface::Private::cancelDrag(quint32 serial)
 {
-    // text input version 0 might call this multiple times
-    if (textInputs.contains(ti)) {
-        return;
+    if (drag.target) {
+        drag.target->updateDragTarget(nullptr, serial);
+        drag.target = nullptr;
     }
-    textInputs << ti;
-    if (textInput.focus.surface && textInput.focus.surface->client() == ti->client()) {
-        // this is a text input for the currently focused text input surface
-        if (!textInput.focus.textInput) {
-            textInput.focus.textInput = ti;
-            ti->d_func()->sendEnter(textInput.focus.surface, textInput.focus.serial);
-            emit q->focusedTextInputChanged();
-        }
-    }
-    QObject::connect(ti, &QObject::destroyed, q,
-        [this, ti] {
-            textInputs.removeAt(textInputs.indexOf(ti));
-            if (textInput.focus.textInput == ti) {
-                textInput.focus.textInput = nullptr;
-                emit q->focusedTextInputChanged();
-            }
-        }
-    );
+    endDrag(serial);
 }
 
 void SeatInterface::Private::endDrag(quint32 serial)
 {
-    auto target = drag.target;
     QObject::disconnect(drag.destroyConnection);
     QObject::disconnect(drag.dragSourceDestroyConnection);
-    if (drag.source && drag.source->dragSource()) {
-        drag.source->dragSource()->dropPerformed();
+
+    DataDeviceInterface *dragTargetDevice = drag.target;
+    DataDeviceInterface *dragSourceDevice = drag.source;
+    DataSourceInterface *dragSource = dragSourceDevice ? dragSourceDevice->dragSource() : nullptr;
+    if (dragSource) {
+        // TODO: Also check the current drag-and-drop action.
+        if (dragTargetDevice && dragSource->isAccepted()) {
+            dragTargetDevice->drop();
+            dragSource->dropPerformed();
+        } else {
+            if (wl_resource_get_version(dragSource->resource()) >=
+                    WL_DATA_SOURCE_DND_FINISHED_SINCE_VERSION) {
+                dragSource->cancel();
+            }
+        }
     }
-    if (target) {
-        target->drop();
-        target->updateDragTarget(nullptr, serial);
+
+    if (dragTargetDevice) {
+        dragTargetDevice->updateDragTarget(nullptr, serial);
     }
+    if (dragSourceDevice) {
+        auto devicePrivate = DataDeviceInterfacePrivate::get(dragSourceDevice);
+        devicePrivate->endDrag();
+    }
+
     drag = Drag();
     emit q->dragSurfaceChanged();
     emit q->dragEnded();
@@ -447,7 +446,7 @@ void SeatInterface::Private::endDrag(quint32 serial)
 void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice)
 {
     // if the update is from the focussed window we should inform the active client
-    if (!(keys.focus.surface && (keys.focus.surface->client() == dataDevice->client()))) {
+    if (!(globalKeyboard.focus.surface && (*globalKeyboard.focus.surface->client() == dataDevice->client()))) {
         return;
     }
     q->setSelection(dataDevice->selection());
@@ -456,7 +455,7 @@ void SeatInterface::Private::updateSelection(DataDeviceInterface *dataDevice)
 void SeatInterface::Private::updatePrimarySelection(PrimarySelectionDeviceV1Interface *primarySelectionDevice)
 {
     // if the update is from the focussed window we should inform the active client
-    if (!(keys.focus.surface && (*keys.focus.surface->client() == primarySelectionDevice->client()))) {
+    if (!(globalKeyboard.focus.surface && (*globalKeyboard.focus.surface->client() == primarySelectionDevice->client()))) {
         return;
     }
     q->setPrimarySelection(primarySelectionDevice->selection());
@@ -465,10 +464,15 @@ void SeatInterface::Private::updatePrimarySelection(PrimarySelectionDeviceV1Inte
 void SeatInterface::setHasKeyboard(bool has)
 {
     Q_D();
-    if (d->keyboard == has) {
+    if (d->keyboard.isNull() != has) {
         return;
     }
-    d->keyboard = has;
+    if (has) {
+        d->keyboard.reset(new KeyboardInterface(this));
+    } else {
+        d->keyboard.reset();
+    }
+
     emit hasKeyboardChanged(d->keyboard);
 }
 
@@ -549,32 +553,12 @@ void SeatInterface::Private::getKeyboardCallback(wl_client *client, wl_resource
 
 void SeatInterface::Private::getKeyboard(wl_client *client, wl_resource *resource, uint32_t id)
 {
-    // TODO: only create if seat has keyboard?
-    KeyboardInterface *keyboard = new KeyboardInterface(q, resource);
-    auto clientConnection = display->getConnection(client);
-    keyboard->create(clientConnection, qMin(wl_resource_get_version(resource), s_keyboardVersion) , id);
-    if (!keyboard->resource()) {
-        wl_resource_post_no_memory(resource);
-        delete keyboard;
+    if (!keyboard) {
         return;
     }
-    keyboard->repeatInfo(keys.keyRepeat.charactersPerSecond, keys.keyRepeat.delay);
-    if (keys.keymap.xkbcommonCompatible) {
-        keyboard->setKeymap(keys.keymap.content);
-    }
-    keyboards << keyboard;
-    if (keys.focus.surface && keys.focus.surface->client() == clientConnection) {
-        // this is a keyboard for the currently focused keyboard surface
-        keys.focus.keyboards << keyboard;
-        keyboard->setFocusedSurface(keys.focus.surface, keys.focus.serial);
-    }
-    QObject::connect(keyboard, &QObject::destroyed, q,
-        [keyboard,this] {
-            keyboards.removeAt(keyboards.indexOf(keyboard));
-            keys.focus.keyboards.removeOne(keyboard);
-        }
-    );
-    emit q->keyboardCreated(keyboard);
+
+    keyboard->d->add(client, id, wl_resource_get_version(resource));
+    emit q->keyboardCreated(keyboard.data());
 }
 
 void SeatInterface::Private::getTouchCallback(wl_client *client, wl_resource *resource, uint32_t id)
@@ -930,13 +914,11 @@ void SeatInterface::pointerButtonPressed(quint32 button)
         for (auto it = d->globalPointer.focus.pointers.constBegin(), end = d->globalPointer.focus.pointers.constEnd(); it != end; ++it) {
             (*it)->buttonPressed(button, serial);
         }
-        if (focusSurface == d->keys.focus.surface) {
+        if (focusSurface == d->globalKeyboard.focus.surface) {
             // update the focused child surface
             auto p = focusedPointer();
-            if (p) {
-                for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-                    (*it)->d_func()->focusChildSurface(p->d_func()->focusedChildSurface, serial);
-                }
+            if (p && d->keyboard) {
+                d->keyboard->d->focusChildSurface(p->d_func()->focusedChildSurface, serial);
             }
         }
     }
@@ -1120,64 +1102,38 @@ void SeatInterface::cancelPointerPinchGesture()
     d->globalPointer.gestureSurface.clear();
 }
 
-void SeatInterface::keyPressed(quint32 key)
-{
-    Q_D();
-    d->keys.lastStateSerial = d->display->nextSerial();
-    if (!d->updateKey(key, Private::Keyboard::State::Pressed)) {
-        return;
-    }
-    if (d->keys.focus.surface) {
-        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-            (*it)->keyPressed(key, d->keys.lastStateSerial);
-        }
-    }
-}
-
-void SeatInterface::keyReleased(quint32 key)
-{
-    Q_D();
-    d->keys.lastStateSerial = d->display->nextSerial();
-    if (!d->updateKey(key, Private::Keyboard::State::Released)) {
-        return;
-    }
-    if (d->keys.focus.surface) {
-        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-            (*it)->keyReleased(key, d->keys.lastStateSerial);
-        }
-    }
-}
-
 SurfaceInterface *SeatInterface::focusedKeyboardSurface() const
 {
     Q_D();
-    return d->keys.focus.surface;
+    return d->globalKeyboard.focus.surface;
 }
 
 void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
 {
     Q_D();
+    if (!d->keyboard) {
+        qCWarning(KWAYLAND_SERVER) << "Can not set focused keyboard surface without keyboard capability";
+        return;
+    }
+
     const quint32 serial = d->display->nextSerial();
-    for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-        (*it)->setFocusedSurface(nullptr, serial);
+
+    if (d->globalKeyboard.focus.surface) {
+        disconnect(d->globalKeyboard.focus.destroyConnection);
     }
-    if (d->keys.focus.surface) {
-        disconnect(d->keys.focus.destroyConnection);
-    }
-    d->keys.focus = Private::Keyboard::Focus();
-    d->keys.focus.surface = surface;
-    d->keys.focus.keyboards = d->keyboardsForSurface(surface);
-    if (d->keys.focus.surface) {
-        d->keys.focus.destroyConnection = connect(surface, &QObject::destroyed, this,
+    d->globalKeyboard.focus = Private::Keyboard::Focus();
+    d->globalKeyboard.focus.surface = surface;
+    if (d->globalKeyboard.focus.surface) {
+        d->globalKeyboard.focus.destroyConnection = connect(surface, &QObject::destroyed, this,
             [this] {
                 Q_D();
-                d->keys.focus = Private::Keyboard::Focus();
+                d->globalKeyboard.focus = Private::Keyboard::Focus();
             }
         );
-        d->keys.focus.serial = serial;
+        d->globalKeyboard.focus.serial = serial;
         // selection?
         const QVector<DataDeviceInterface *> dataDevices = d->dataDevicesForSurface(surface);
-        d->keys.focus.selections = dataDevices;
+        d->globalKeyboard.focus.selections = dataDevices;
         for (auto dataDevice : dataDevices) {
             if (d->currentSelection) {
                 dataDevice->sendSelection(d->currentSelection);
@@ -1193,7 +1149,7 @@ void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
             }
         }
 
-        d->keys.focus.primarySelections = primarySelectionDevices;
+        d->globalKeyboard.focus.primarySelections = primarySelectionDevices;
         for (auto primaryDataDevice : primarySelectionDevices) {
             if (d->currentSelection) {
                 primaryDataDevice->sendSelection(d->currentPrimarySelection);
@@ -1202,156 +1158,19 @@ void SeatInterface::setFocusedKeyboardSurface(SurfaceInterface *surface)
             }
         }
     }
-    for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-        (*it)->setFocusedSurface(surface, serial);
-    }
+
+    d->keyboard->setFocusedSurface(surface, serial);
+
     // focused text input surface follows keyboard
     if (hasKeyboard()) {
         setFocusedTextInputSurface(surface);
     }
 }
 
-#if KWAYLANDSERVER_BUILD_DEPRECATED_SINCE(5, 69)
-void SeatInterface::setKeymap(int fd, quint32 size)
-{
-    QFile file;
-    if (!file.open(fd, QIODevice::ReadOnly)) {
-        return;
-    }
-    const char *address = reinterpret_cast<char*>(file.map(0, size));
-    if (!address) {
-        return;
-    }
-    setKeymapData(QByteArray(address, size));
-}
-#endif
-
-void SeatInterface::setKeymapData(const QByteArray &content)
+KeyboardInterface *SeatInterface::keyboard() const
 {
     Q_D();
-    d->keys.keymap.xkbcommonCompatible = true;
-    d->keys.keymap.content = content;
-    for (auto it = d->keyboards.constBegin(); it != d->keyboards.constEnd(); ++it) {
-        (*it)->setKeymap(content);
-    }
-}
-
-void SeatInterface::updateKeyboardModifiers(quint32 depressed, quint32 latched, quint32 locked, quint32 group)
-{
-    Q_D();
-    bool changed = false;
-#define UPDATE( value ) \
-    if (d->keys.modifiers.value != value) { \
-        d->keys.modifiers.value = value; \
-        changed = true; \
-    }
-    UPDATE(depressed)
-    UPDATE(latched)
-    UPDATE(locked)
-    UPDATE(group)
-    if (!changed) {
-        return;
-    }
-    const quint32 serial = d->display->nextSerial();
-    d->keys.modifiers.serial = serial;
-    if (d->keys.focus.surface) {
-        for (auto it = d->keys.focus.keyboards.constBegin(), end = d->keys.focus.keyboards.constEnd(); it != end; ++it) {
-            (*it)->updateModifiers(depressed, latched, locked, group, serial);
-        }
-    }
-}
-
-void SeatInterface::setKeyRepeatInfo(qint32 charactersPerSecond, qint32 delay)
-{
-    Q_D();
-    d->keys.keyRepeat.charactersPerSecond = qMax(charactersPerSecond, 0);
-    d->keys.keyRepeat.delay = qMax(delay, 0);
-    for (auto it = d->keyboards.constBegin(); it != d->keyboards.constEnd(); ++it) {
-        (*it)->repeatInfo(d->keys.keyRepeat.charactersPerSecond, d->keys.keyRepeat.delay);
-    }
-}
-
-qint32 SeatInterface::keyRepeatDelay() const
-{
-    Q_D();
-    return d->keys.keyRepeat.delay;
-}
-
-qint32 SeatInterface::keyRepeatRate() const
-{
-    Q_D();
-    return d->keys.keyRepeat.charactersPerSecond;
-}
-
-bool SeatInterface::isKeymapXkbCompatible() const
-{
-    Q_D();
-    return d->keys.keymap.xkbcommonCompatible;
-}
-
-#if KWAYLANDSERVER_BUILD_DEPRECATED_SINCE(5, 69)
-int SeatInterface::keymapFileDescriptor() const
-{
-    return -1;
-}
-#endif
-
-#if KWAYLANDSERVER_BUILD_DEPRECATED_SINCE(5, 69)
-quint32 SeatInterface::keymapSize() const
-{
-    return 0;
-}
-#endif
-
-quint32 SeatInterface::depressedModifiers() const
-{
-    Q_D();
-    return d->keys.modifiers.depressed;
-}
-
-quint32 SeatInterface::groupModifiers() const
-{
-    Q_D();
-    return d->keys.modifiers.group;
-}
-
-quint32 SeatInterface::latchedModifiers() const
-{
-    Q_D();
-    return d->keys.modifiers.latched;
-}
-
-quint32 SeatInterface::lockedModifiers() const
-{
-    Q_D();
-    return d->keys.modifiers.locked;
-}
-
-quint32 SeatInterface::lastModifiersSerial() const
-{
-    Q_D();
-    return d->keys.modifiers.serial;
-}
-
-QVector< quint32 > SeatInterface::pressedKeys() const
-{
-    Q_D();
-    QVector<quint32> keys;
-    for (auto it = d->keys.states.constBegin(); it != d->keys.states.constEnd(); ++it) {
-        if (it.value() == Private::Keyboard::State::Pressed) {
-            keys << it.key();
-        }
-    }
-    return keys;
-}
-
-KeyboardInterface *SeatInterface::focusedKeyboard() const
-{
-    Q_D();
-    if (d->keys.focus.keyboards.isEmpty()) {
-        return nullptr;
-    }
-    return d->keys.focus.keyboards.first();
+    return d->keyboard.data();
 }
 
 void SeatInterface::cancelTouchSequence()
@@ -1361,14 +1180,8 @@ void SeatInterface::cancelTouchSequence()
         (*it)->cancel();
     }
     if (d->drag.mode == Private::Drag::Mode::Touch) {
-        // cancel the drag, don't drop.
-        if (d->drag.target) {
-            // remove the current target
-            d->drag.target->updateDragTarget(nullptr, 0);
-            d->drag.target = nullptr;
-        }
-        // and end the drag for the source, serial does not matter
-        d->endDrag(0);
+        // cancel the drag, don't drop. serial does not matter
+        d->cancelDrag(0);
     }
     d->globalTouch.ids.clear();
 }
@@ -1610,50 +1423,48 @@ void SeatInterface::setFocusedTextInputSurface(SurfaceInterface *surface)
 {
     Q_D();
     const quint32 serial = d->display->nextSerial();
-    const auto old = d->textInput.focus.textInput;
-    if (d->textInput.focus.textInput) {
-        // TODO: setFocusedSurface like in other interfaces
-        d->textInput.focus.textInput->d_func()->sendLeave(serial, d->textInput.focus.surface);
+
+    if (d->focusedTextInputSurface) {
+        disconnect(d->focusedSurfaceDestroyConnection);
     }
-    if (d->textInput.focus.surface) {
-        disconnect(d->textInput.focus.destroyConnection);
+
+    if (d->focusedTextInputSurface != surface){
+        d->textInputV2->d->sendLeave(serial, d->focusedTextInputSurface);
+        d->textInputV3->d->sendLeave(d->focusedTextInputSurface);
+        d->focusedTextInputSurface = surface;
+        emit focusedTextInputSurfaceChanged();
     }
-    d->textInput.focus = Private::TextInput::Focus();
-    d->textInput.focus.surface = surface;
-    TextInputInterface *t = d->textInputForSurface(surface);
-    if (t && !t->resource()) {
-        t = nullptr;
-    }
-    d->textInput.focus.textInput = t;
-    if (d->textInput.focus.surface) {
-        d->textInput.focus.destroyConnection = connect(surface, &SurfaceInterface::aboutToBeDestroyed, this,
+
+    if (d->focusedTextInputSurface) {
+        d->focusedSurfaceDestroyConnection = connect(surface, &SurfaceInterface::aboutToBeDestroyed, this,
             [this] {
                 setFocusedTextInputSurface(nullptr);
             }
         );
-        d->textInput.focus.serial = serial;
-    }
-    if (t) {
-        // TODO: setFocusedSurface like in other interfaces
-        t->d_func()->sendEnter(surface, serial);
-    }
-    if (old != t) {
-        emit focusedTextInputChanged();
     }
+
+    d->textInputV2->d->sendEnter(surface, serial);
+    d->textInputV3->d->sendEnter(surface);
+    // TODO: setFocusedSurface like in other interfaces
 }
 
 SurfaceInterface *SeatInterface::focusedTextInputSurface() const
 {
     Q_D();
-    return d->textInput.focus.surface;
+    return d->focusedTextInputSurface;
 }
 
-TextInputInterface *SeatInterface::focusedTextInput() const
+TextInputV2Interface *SeatInterface::textInputV2() const
 {
     Q_D();
-    return d->textInput.focus.textInput;
+    return d->textInputV2;
 }
 
+TextInputV3Interface *SeatInterface::textInputV3() const
+{
+    Q_D();
+    return d->textInputV3;
+}
 AbstractDataSource *SeatInterface::selection() const
 {
     Q_D();
@@ -1676,13 +1487,12 @@ void SeatInterface::setSelection(AbstractDataSource *selection)
         auto cleanup = [this]() {
             setSelection(nullptr);
         };
-        connect(selection, &DataSourceInterface::unbound, this, cleanup);
-        connect(selection, &QObject::destroyed, this, cleanup);
+        connect(selection, &DataSourceInterface::aboutToBeDestroyed, this, cleanup);
     }
 
     d->currentSelection = selection;
 
-    for (auto focussedSelection: qAsConst(d->keys.focus.selections)) {
+    for (auto focussedSelection: qAsConst(d->globalKeyboard.focus.selections)) {
         if (selection) {
             focussedSelection->sendSelection(selection);
         } else {
@@ -1716,12 +1526,12 @@ void SeatInterface::setPrimarySelection(AbstractDataSource *selection)
         auto cleanup = [this]() {
             setPrimarySelection(nullptr);
         };
-        connect(selection, &DataSourceInterface::unbound, this, cleanup);
+        connect(selection, &DataSourceInterface::aboutToBeDestroyed, this, cleanup);
     }
 
     d->currentPrimarySelection = selection;
 
-    for (auto focussedSelection: qAsConst(d->keys.focus.primarySelections)) {
+    for (auto focussedSelection: qAsConst(d->globalKeyboard.focus.primarySelections)) {
         if (selection) {
             focussedSelection->sendSelection(selection);
         } else {
